@@ -142,6 +142,20 @@ class Form(object):
         """This is the initial data loaded from the given item. Used to
         render the readonly forms"""
 
+    def _set_current_field_data(self, data):
+        for key in self.fields:
+            value = data.get(key)
+            if value or isinstance(value, int):
+                field = self.fields[key]
+                field.set_value(value)
+
+    def _set_previous_field_data(self, data):
+        for key in self.fields:
+            value = data.get(key)
+            if value or isinstance(value, int):
+                field = self.fields[key]
+                field.set_previous_value(value)
+
     def _get_data_from_item(self):
         values = {}
         if not self._item:
@@ -281,7 +295,7 @@ class Form(object):
         return self.external_validators.append(validator)
 
     def render(self, values={}, page=0, buttons=True,
-               previous_values=None, outline=True):
+               previous_values={}, outline=True):
         """Returns the rendererd form as an HTML string.
 
         :values: Dictionary with values to be prefilled/overwritten in
@@ -304,13 +318,15 @@ class Form(object):
         # values will overwrite the item_values.
         values = dict(item_values.items() + values.items())
 
+        # Set current and previous values of the fields in the form.
+        self._set_current_field_data(values)
+        self._set_previous_field_data(previous_values)
+
         # Add csrf_token to the values dictionary
         values['csrf_token'] = self._csrf_token
 
         renderer = FormRenderer(self, self._translate)
-        form = renderer.render(values=values, buttons=buttons,
-                               previous_values=previous_values,
-                               outline=outline)
+        form = renderer.render(buttons=buttons, outline=outline)
         # TODO: Can happen: Error while parsing form in htmlfill: %s. Hint See
         # 'https://github.com/formencode/formencode/issues/57desc (ti)
         # <2014-03-21 10:29>
@@ -636,6 +652,7 @@ class Field(object):
         self.renderer = get_renderer(self, translate)
         self._errors = []
         self._warnings = []
+
         # Set default value
         value = getattr(self._config, "value")
         if value and value.startswith("$"):
@@ -649,6 +666,7 @@ class Field(object):
                 log.error("Error while accessing attribute '%s': %s" % (value, e))
                 value = None
         self.value = value
+
         self.previous_value = None
         """Value as string of the field. Will be set on rendering the
         form"""
@@ -767,11 +785,9 @@ class Field(object):
 
     def _load_options_from_db(self):
         # Get mapped clazz for the field
-        options = []
-        if self.get_type() == 'manytoone':
-            options.append(("None", "", True))
         try:
             clazz = self._get_sa_mapped_class()
+            return self._form._dbsession.query(clazz)
         except:
             # Catch exception here. This exception can happen when
             # rendering the form in the preview of the formeditor. In
@@ -779,20 +795,22 @@ class Field(object):
             # class.
             log.error("Can not get a mappend class for '%s' "
                       "to load the option from db" % self.name)
-            return options
-        items = self._form._dbsession.query(clazz)
-        for item in items:
+        return []
+
+    def filter_options(self, options):
+        filtered_options = []
+        for option in options:
             if self._config.renderer and self._config.renderer.filter:
                 expr = self._build_filter_expr(
-                    self._config.renderer.filter, item)
+                    self._config.renderer.filter, option)
                 rule = Rule(expr)
                 if rule.evaluate({}):
-                    options.append((item, item.id, True))
+                    filtered_options.append((option, option.id, True))
                 else:
-                    options.append((item, item.id, False))
+                    filtered_options.append((option, option.id, False))
             else:
-                options.append((item, item.id, True))
-        return options
+                filtered_options.append((option, option.id, True))
+        return filtered_options
 
     def get_options(self):
         """Will return a list of tuples containing the options of the
@@ -813,13 +831,15 @@ class Field(object):
         the SQLAlchemy model and which are loaded from the database.
         """
         options = []
+        if self.get_type() == 'manytoone':
+            options.append(("None", "", True))
         user_defined_options = self._config.options
         if user_defined_options:
             for option in user_defined_options:
                 # TODO: Filter user defined options too (ti) <2014-02-19 23:46>
                 options.append((option[0], option[1], True))
         elif self._form._dbsession:
-            return self._load_options_from_db()
+            options.extend(self.filter_options(self._load_options_from_db()))
         else:
             # TODO: Try to get the session from the item. Ther must be
             # somewhere the already bound session. (torsten) <2013-07-23 00:27>
