@@ -4,8 +4,7 @@ import difflib
 from mako.lookup import TemplateLookup
 from formbar import template_dir
 
-template_lookup = TemplateLookup(directories=[template_dir],
-                                 module_directory='/tmp/phormular_modules')
+template_lookup = TemplateLookup(directories=[template_dir])
 
 log = logging.getLogger(__name__)
 
@@ -34,6 +33,10 @@ def get_renderer(field, translate):
             return DropdownFieldRenderer(field, translate)
         elif renderer.render_type == "selection":
             return SelectionFieldRenderer(field, translate)
+        elif renderer.render_type == "radio":
+            return RadioFieldRenderer(field, translate)
+        elif renderer.render_type == "checkbox":
+            return CheckboxFieldRenderer(field, translate)
         elif renderer.render_type == "datepicker":
             return DateFieldRenderer(field, translate)
         elif renderer.render_type == "password":
@@ -91,27 +94,19 @@ class FormRenderer(Renderer):
         self.translate = translate
         self.template = template_lookup.get_template("form.mako")
 
-    def render(self, buttons=True, values=None,
-               previous_values=None, outline=True):
+    def render(self, buttons=True, outline=True):
         """Returns the rendered form as string.
 
         :buttons: Boolean flag to indicate if the form buttons should be
         rendererd. Defaults to true.
-        :previous_values: Dictionary of values of the last saved state
-        of the item. If provided a diff between the current and previous
-        values will be renderered in readonly mode.
         :outline: Boolean flag to indicate that the outline for pages
         should be rendered. Defaults to true.
         :returns: rendered form.
 
         """
-        if not values:
-            values = {}
-        if not previous_values:
-            previous_values = {}
         html = []
         html.append(self._render_form_start())
-        html.append(self._render_form_body(values, previous_values, outline))
+        html.append(self._render_form_body(outline))
         if not self._form._config.readonly and buttons:
             html.append(self._render_form_buttons())
         html.append(self._render_form_end())
@@ -138,12 +133,10 @@ class FormRenderer(Renderer):
                         % self._form._csrf_token)
         return "".join(html)
 
-    def _render_form_body(self, values, previous_values, render_outline):
+    def _render_form_body(self, render_outline):
         values = {'form': self._form,
                   '_': self.translate,
-                  'render_outline': render_outline,
-                  'previous_values': previous_values,
-                  'values': values}
+                  'render_outline': render_outline}
         return self.template.render(**values)
 
     def _render_form_buttons(self):
@@ -154,10 +147,32 @@ class FormRenderer(Renderer):
             html.append('<div class="col-sm-9 span9 button-pane">')
         else:
             html.append('<div class="col-sm-12 span12 button-pane well-small">')
-        html.append('<button type="submit" '
-                    'class="btn btn-default">%s</button>' % 'Submit')
-        html.append('<button type="reset" '
-                    'class="btn btn-default">%s</button>' % 'Reset')
+
+        # Render default buttons if no buttons have been defined for the
+        # form.
+        if len(self._form._config._buttons) == 0:
+            html.append('<button type="submit" '
+                        'class="btn btn-default">%s</button>' % 'Submit')
+            html.append('<button type="reset" '
+                        'class="btn btn-default">%s</button>' % 'Reset')
+        else:
+            for b in self._form._config._buttons:
+                if b.attrib.get("icon"):
+                    icon = '<i class="%s"/>' % b.attrib.get("icon")
+                else:
+                    icon = ""
+                attr = {
+                    'type': b.attrib.get("type") or "submit",
+                    'value': b.attrib.get("value") or "",
+                    'class': "btn btn-%s" % (b.attrib.get("class") or "default"),
+                    'icon': icon,
+                    'label': b.text or "XXX"
+                }
+                html.append('<button type="%(type)s" name="_%(type)s"'
+                            ' value="%(value)s" class="%(class)s">'
+                            '%(icon)s %(label)s</button>'
+                            % (attr))
+        # Else render defined buttons.
         html.append('</div>')
         html.append('</div>')
         return "".join(html)
@@ -189,6 +204,8 @@ class FieldRenderer(Renderer):
 
     def __getattr__(self, name):
         """Give access to the config values of the renderer"""
+        if self._config is None:
+            return None
         return getattr(self._config, name)
 
     def _render_label(self):
@@ -264,13 +281,37 @@ class FieldRenderer(Renderer):
         html = []
         has_errors = len(self._field.get_errors())
         has_warnings = len(self._field.get_warnings())
-        html.append('<div class="form-group %s %s">' % ((has_errors and 'has-error'),
-                                                       (has_warnings and 'has-warning')))
-        html.append(self._render_label())
+        html.append('<div class="form-group %s %s">' % ((has_errors and 'has-error'), (has_warnings and 'has-warning')))
         values = self._get_template_values()
-        html.append(self.template.render(**values))
-        html.append(self._render_errors())
-        html.append(self._render_help())
+        if self.label_width > 0 and self.label_position in ["left", "right"]:
+            label_width = self.label_width
+            label_align = self.label_align
+            field_width = 12-self.label_width
+            html.append('<div class="row">')
+            if self.label_position == "left":
+                html.append('<div class="col-sm-%s" align="%s">' % (label_width, label_align))
+                html.append(self._render_label())
+                html.append('</div>')
+                html.append('<div class="col-sm-%s">' % field_width)
+                html.append(self.template.render(**values))
+                html.append(self._render_errors())
+                html.append(self._render_help())
+                html.append('</div>')
+            else:
+                html.append('<div class="col-sm-%s">' % field_width)
+                html.append(self.template.render(**values))
+                html.append(self._render_errors())
+                html.append(self._render_help())
+                html.append('</div>')
+                html.append('<div class="col-sm-%s" align="%s">' % (label_width, label_align))
+                html.append(self._render_label())
+                html.append('</div>')
+            html.append("</div>")
+        else:
+            html.append(self._render_label())
+            html.append(self.template.render(**values))
+            html.append(self._render_errors())
+            html.append(self._render_help())
         html.append('</div>')
         return "".join(html)
 
@@ -355,17 +396,14 @@ class HTMLRenderer(FieldRenderer):
         return "".join(html)
 
 
-# TODO: Use a new superclass for the following two renderers as they
-# deal not only with one single value but with multiple selectable
-# values. Implement filtering of items here if possible. See Plorma
-# implementation of the ListingFieldRenderer to see how this ignoring is
-# implemented. (ti) <2013-10-11 22:39>
-class DropdownFieldRenderer(FieldRenderer):
-    """A Renderer to render dropdown list"""
-
+class OptionFieldRenderer(FieldRenderer):
+    """Superclass for fields element which supports selecting one or
+    more options from a selection"""
+    # TODO: Implement filtering of items here if possible. See Plorma
+    # implementation of the ListingFieldRenderer to see how this
+    # ignoring is implemented. (ti) <2013-10-11 22:39>
     def __init__(self, field, translate):
         FieldRenderer.__init__(self, field, translate)
-        self.template = template_lookup.get_template("dropdown.mako")
         self._cache_options = None
 
     def _get_template_values(self):
@@ -376,13 +414,34 @@ class DropdownFieldRenderer(FieldRenderer):
         values['options'] = self._cache_options
         return values
 
-class SelectionFieldRenderer(FieldRenderer):
+class DropdownFieldRenderer(OptionFieldRenderer):
+    """A Renderer to render dropdown list"""
+
+    def __init__(self, field, translate):
+        OptionFieldRenderer.__init__(self, field, translate)
+        self.template = template_lookup.get_template("dropdown.mako")
+
+class SelectionFieldRenderer(OptionFieldRenderer):
     """A Renderer to render selection field"""
 
     def __init__(self, field, translate):
-        FieldRenderer.__init__(self, field, translate)
+        OptionFieldRenderer.__init__(self, field, translate)
         self.template = template_lookup.get_template("selection.mako")
 
+class RadioFieldRenderer(OptionFieldRenderer):
+    """A Renderer to render selection field"""
+
+    def __init__(self, field, translate):
+        OptionFieldRenderer.__init__(self, field, translate)
+        self.template = template_lookup.get_template("radio.mako")
+
+class CheckboxFieldRenderer(OptionFieldRenderer):
+    """A Renderer to render selection field"""
+
+    def __init__(self, field, translate):
+        OptionFieldRenderer.__init__(self, field, translate)
+        self.template = template_lookup.get_template("checkbox.mako")
+
 class FormbarEditorRenderer(FieldRenderer):
     """A Renderer to render the formbar editor widget used to edit
     formbar form definitons."""
@@ -401,19 +460,3 @@ class ListFieldRenderer(FieldRenderer):
     def __init__(self, field, translate):
         FieldRenderer.__init__(self, field, translate)
         self.template = template_lookup.get_template("fa_field.mako")
-
-
-class CheckboxFieldRenderer(FieldRenderer):
-    """A Renderer to render checkboxes"""
-
-    def __init__(self, field, translate):
-        FieldRenderer.__init__(self, field, translate)
-        self.template = template_lookup.get_template("fa_field.mako")
-
-
-class RadioFieldRenderer(FieldRenderer):
-    """A Renderer to render radio boxes"""
-
-    def __init__(self, field, translate):
-        FieldRenderer.__init__(self, field, translate)
-        self.template = template_lookup.get_template("fa_field.mako")
