@@ -37,6 +37,11 @@ class TestParseRule(unittest.TestCase):
         result = self.parser.parse(expr)
         self.assertEqual(expr, "".join(result.asList()))
 
+    def test_lteq_expr(self):
+        expr = "2 le 4" # -> "2<=4"
+        result = self.parser.parse(expr)
+        self.assertEqual("2<=4", "".join(result.asList()))
+
     def test_gteq_expr(self):
         expr = "$field>=$field"
         result = self.parser.parse(expr)
@@ -47,6 +52,36 @@ class TestParseRule(unittest.TestCase):
         result = self.parser.parse(expr)
         self.assertEqual(expr, " ".join(result.asList()))
 
+    def test_plus_expr(self):
+        expr = "$field + $field"
+        result = self.parser.parse(expr)
+        self.assertEqual(expr, " ".join(result.asList()))
+
+    def test_minus_expr(self):
+        expr = "$field - $field"
+        result = self.parser.parse(expr)
+        self.assertEqual(expr, " ".join(result.asList()))
+
+    def test_mul_expr(self):
+        expr = "$field * $field"
+        result = self.parser.parse(expr)
+        self.assertEqual(expr, " ".join(result.asList()))
+
+    def test_div_expr(self):
+        expr = "$field / $field"
+        result = self.parser.parse(expr)
+        self.assertEqual(expr, " ".join(result.asList()))
+
+    def test_proz_expr(self):
+        expr = "$field / 100 * 10"
+        result = self.parser.parse(expr)
+        self.assertEqual(expr, " ".join(result.asList()))
+
+    def test_grouping_expr(self):
+        expr = "( $do_zg_ap + $do_zg_pxa + $do_zg_team + $do_zg_leit + $do_zg_doz ) < 100"
+        result = self.parser.parse(expr)
+        self.assertEqual(expr, " ".join(result.asList()))
+
     # Test function
 
     def test_function_len_expr(self):
@@ -59,6 +94,16 @@ class TestParseRule(unittest.TestCase):
         result = self.parser.parse(expr)
         self.assertEqual('_bool($field)', "".join(result.asList()))
 
+    def test_function_bool_expr2(self):
+        expr = "bool( '_' )"
+        result = self.parser.parse(expr)
+        self.assertEqual("_bool('_')", "".join(result.asList()))
+
+    def test_function_bool_expr3(self):
+        expr = "bool( 'xxx' )"
+        result = self.parser.parse(expr)
+        self.assertEqual("_bool('xxx')", "".join(result.asList()))
+
 
 class TestEvaluateRule(unittest.TestCase):
 
@@ -146,6 +191,12 @@ class TestEvaluateRule(unittest.TestCase):
         rule = self.build_rule(expr)
         self.assertEqual(rule.evaluate(values), True)
 
+    def test_lteq2_expr_ok(self):
+        values = {}
+        expr = " 2 le 4"
+        rule = self.build_rule(expr)
+        self.assertEqual(rule.evaluate(values), True)
+
     def test_lteq_expr_fail(self):
         values = {"field": 6}
         expr = "$field<=5"
@@ -164,6 +215,12 @@ class TestEvaluateRule(unittest.TestCase):
         rule = self.build_rule(expr)
         self.assertEqual(rule.evaluate(values), False)
 
+    def test_bool_ok(self):
+        values = {"field": "we have a value"}
+        expr = "bool('xxx')"
+        rule = self.build_rule(expr)
+        self.assertEqual(rule.evaluate(values), True)
+
     def test_required_ok(self):
         values = {"field": "we have a value"}
         expr = "bool($field)"
